
# Creation:  6/2016
# Author: Nicola Cortesi
# Aim: Measure the impact of Weather Regimes of a chosen variable, from a reanalysis of from a forecast systems.
#
# I/O: input are all the various "*_psl.RData" created by the script 'weather_regimes.R'
#      its output are "*_<varname>.RData" files which are need by the script weather_regimes_maps.R
#      You can also run this script from terminal with: Rscript ~/scripts/weather_regimes_impact.R
#
# Assumption: all input/output files are located in the 'workdir' folder. Note that this script should not be run with many parallel jobs since it spends most of its time
#      in loading data, so it's better to take advantage of the parallel loading feature of Load() to run the script once computing the monthly analysis in a sequential way
#
# Branch: weather_regimes

rm(list=ls())
library(s2dverification)  # for the function Load()
library(abind)
library(reshape2) # after each undate of s2dverification, it's better to remove and install this package again because sometimes it gets broken! 
#library(TTR)
source('/home/Earth/ncortesi/scripts/Rfunctions.R') # for the calendar functions
#workdir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes"       # working dir
# module load NCO # : load it from the terminal before running this script interactively in the SMP machine, if you didn't load it in the .bashrc

#rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/41_ERA-Interim_monthly_1981-2015_LOESS_filter"
#ean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/18) as 12) but with no lat correction"
#rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/18_as_12_but_with_no_lat_correction"
#rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/44_as_43_but_with_no_lat_correction"
#rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/50_NCEP_monthly_1981-2016_LOESS_filter_lat_corr"
#rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/52_JRA55_monthly_1981-2016_LOESS_filter_lat_corr"
rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/56_JRA55_monthly_1981-2016_LOESS_filter_lat_corr_ordered4variance"
rean.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/EDPR_February_2017"

forecast.dir <- "/scratch/Earth/ncortesi/RESILIENCE/Regimes/42_ECMWF_S4_monthly_1981-2015_LOESS_filter"

# available reanalysis for var data:
#ERAint <- '/esnas/reconstructions/ecmwf/eraint/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
ERAint <- '/esarchive/old-files/recon_ecmwf_erainterim/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
ERAint.name <- "ERA-Interim"

JRA55 <- '/esnas/recon/jma/jra55/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
JRA55.name <- "JRA-55"

NCEP <- '/esnas/recon/noaa/ncep-reanalysis/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'
NCEP.name <- "NCEP"

rean <- JRA55 #JRA55 #ERAint #NCEP # choose one of the two above reanalysis from where to load the input psl data

# available forecast systems for the psl and var data:
#ECMWF_S4 <- list(path = paste0('/esnas/exp/ECMWF/seasonal/0001/s004/m001/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'))
#ECMWF_S4 <- '/esnas/exp/ECMWF/seasonal/0001/s004/m001/$STORE_FREQ$_mean/psl_f6h/psl_$START_DATE$.nc'
#ECMWF_S4 <- '/esnas/exp/ecmwf/system4_m1/$STORE_FREQ$_mean/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'  
ECMWF_S4 <- '/esarchive/old-files/exp_ecmwf_system4_m1/$STORE_FREQ$_mean/old/$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'  # only for daily tas S4

ECMWF_S4.name <- "ECMWF-S4"
member.name <- "ensemble"         # name of the dimension with the ensemble members inside the netCDF files of S4
lat.name <- "latitude"
lon.name <- "longitude"

ECMWF_monthly <- '/esnas/exp/ECMWF/monthly/ensforhc/6hourly/$VAR_NAME$/2014$MONTH$$DAY$00/$VAR_NAME$_$START_DATE$00.nc'
ECMWF_monthly.name <- "ECMWF-Monthly"

forecast <- ECMWF_S4

# set it to 'rean' to load pressure fields and var from reanalisis, or to 'forecast' to load them from forecast systems:
fields <- rean #forecast #rean 

var.num <- 1                                     # Choose a variable. 1: sfcWind  2: tas
var.name <- c("sfcWind","tas")                   # name of the 'predictand' variable of the chosen reanalysis
var.name.full <- c("wind Speed","temperature")   # full name of the 'predictand' variable to put in the title of the graphs
var.unit <- c("m/s", "ºC")                       # unit of measure of var (for drawing color scales)

WR.period <- 2 #1:12 #13:16  # 1-12: Jan-Dec; 13-16: Winter-Spring-Summer-Autumn  # For forecasts, it is the start month

# the following variables have to be the same as in the weather_regimes_vXX.R script that processed the psl data:
year.start <- 1981 #1979 #1981 #1982 #1981       # specify the first year of var data 
year.end <- 2017 #2016 #2015 #2013 #2010         # specify the last year of var data

# Only for seasonal forecasts:
startM <- 3                                      # start month (1=January, 12=December)  [bypassed by the optional arguments of the script]
leadM <- 6                                       # select only the data of one lead month:  [bypassed by the optional arguments of the script]
n.members <- 15                                  # number of members of the forecast system to load
n.leadtimes <- 216                               # number of leadtimes of the forecast system to load
missing.forecasts=FALSE     # set it to TRUE if there can be missing hindcasts files in the forecast var data, so it will load only the available years and deals with NA
res <- 0.75                # set the resolution you want to interpolate the var data when loading it (i.e: set to 0.75 to use the same res of ERA-Interim)
                           # interpolation method is BILINEAR.

# Coordinates of the box enclosing the study region (the Northern Atlantic in this case):
lat.max <- 81 #81 #80 #70
lat.min <- 27 #30 #20 #30
lon.max <- 45 #36 #30 #40 
lon.min <- 274.5 #274.5 #270 #280 # put a positive number here because the geopotential has only positive vaues of longitud!

#############################################################################################################################
#ECMWF_monthly <- list(path = paste0('/esnas/exp/ECMWF/monthly/ensforhc/6hourly/psl/2014010200/1994_010200.nc'))
#ECMWF_monthly <- list(path = paste0('/esnas/exp/ECMWF/monthly/ensforhc/6hourly/$VAR_NAME$/2014$MONTH$$DAY$00/$VAR_NAME$_$START_DATE$00.nc'))

rean.name <- unname(ifelse(rean == ERAint, ERAint.name, ifelse(rean == JRA55, JRA55.name, NCEP.name)))
forecast.name <- unname(ifelse(forecast == ECMWF_S4, ECMWF_S4.name, ECMWF_monthly.name))
fields.name <- unname(ifelse(fields == rean, rean.name, forecast.name))

# in case the script is run with one argument, it is assumed a reanalysis is being used and the argument becomes the month we want to perform the clustering;
# in case the script is run with two arguments, it is assumed a forecast is used and the first argument represents the startmonth and the second one the leadmonth.
# in case the script is run with no arguments, the values of the variables inside the script are used:
script.arg <- as.integer(commandArgs(TRUE))

# in case the script is run with no arguments:
if(length(script.arg) == 0 && fields.name == forecast.name){
   start.month <- startM
   #WR.period <- start.month
   lead.month <- leadM
}

# in case the script is run with 1 argument, it is assumed you are using a reanalysis, so the variable to change is the period:
if(length(script.arg) == 1){
   fields <- rean
   fields.name <- rean.name
   WR.period <-  script.arg[1]      
}

# in case the script is run with 2 arguments, it is assumed you are using forecasts, so the variables to change are the start date and the lead time:
if(length(script.arg) >= 2){
   fields <- forecast
   fields.name <- forecast.name
   start.month <- script.arg[1]                 
   lead.month <- script.arg[2]
   WR.period <- start.month  # it becomes just a copy of the start.month
}

if(length(script.arg) == 3){ # in this case, we are running the script in the SMP machine and we need to override the variables below:
   source('/gpfs/projects/bsc32/bsc32842/Rfunctions.R') # for the calendar function
   #workdir <- "/gpfs/projects/bsc32/bsc32842/RESILIENCE"  # working dir
}

if(length(script.arg) >= 2) {lead.comment <- paste0("Lead month: ", lead.month)} else {lead.comment <- ""}

cat(paste0("Field: ", fields.name, " Period: ", WR.period, " ", lead.comment, "\n"))

#if(fields.name == forecast.name) WR.period = 1

#if(length(script.arg) == 3){ # in this case, we are running the script in the SMP machine and we need to override the variables below:
#   source('/gpfs/projects/bsc32/bsc32842/Rfunctions.R') # for the calendar function
#   workdir <- "/gpfs/projects/bsc32/bsc32842/RESILIENCE"  # working dir
#}

#if(length(script.arg) >= 2) {lead.comment <- paste0("Lead month: ", lead.month)} else {lead.comment <- ""}
#cat(paste0("Field: ", fields.name, " Period: ", WR.period, " ", lead.comment, "\n"))


# Create a regular lat/lon grid to interpolate the data to the chosen res: (Notice that the regular grid is always centered at lat=0 and lon=0!)
n.lon.grid <- 360/res
n.lat.grid <- (180/res)+1
my.grid <- paste0('r',n.lon.grid,'x',n.lat.grid)

## days.period <- n.days.period <- period.length <- list()
## for (pp in 1:17){
##     days.period[[pp]] <- NA
##     for(y in year.start:year.end) days.period[[pp]] <- c(days.period[[pp]], n.days.in.a.future.year(year.start, y) + pos.period(y,pp))
##     days.period[[pp]] <- days.period[[pp]][-1] # remove the NA at the begin that was introduced only to be able to execute the above command
##     # number of days belonging to that period from year.start to year.end:
##     n.days.period[[pp]] <- length(days.period[[pp]])
##     # Number of days belonging to that period in a single year: 
##     period.length[[pp]] <- n.days.in.a.period(pp,1999) #+ ifelse(period==13 | period==2, 1, 0)  # using year 1999 introduce a small error in winter season length because of bisestile years
## }


# Load var data (interpolating it to the same reanalysis grid, if necessary):
cat("Loading data. Please wait...\n")

# Load var data:
if(fields.name == rean.name){ # Load daily var data in the reanalysis case:
    info.period <- WR.period[1]  # period used to get the variables n.years, lat.min, lat.max, lon.min y lon.max
    WR.period.old <- WR.period
    
    load(file=paste0(rean.dir,"/",fields.name,"_",my.period[info.period],"_psl.RData"))
    if(!identical(WR.period, WR.period.old)) WR.period <- WR.period.old
    
    n.years <- year.end - year.start + 1
    
    #for (y in year.start:year.end){
    #    var <- Load(var = var.name[var.num], exp = NULL, obs = list(fields), paste0(y,'0101'), storefreq = 'daily', leadtimemax = n.days.in.a.year(y), output = 'lonlat',
    #                latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max)
    #    vareuFull[seq.days.in.a.future.year(year.start, y),,] <- var$obs
    #    rm(var)
    #    gc()
    #}

    vareuFull366 <-  Load(var = var.name[var.num], exp = NULL, obs = list(list(path=fields)), paste0(year.start:year.end,'0101'), storefreq = 'daily', leadtimemax = 366, output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max, nprocs=8)$obs

    vareuFull <-array(NA,c(1,1,n.years,365, dim(vareuFull366)[5], dim(vareuFull366)[6]))
    
    # remove bisestile days to have all arrays with the same dimensions:
    cat("Removing bisestile days. Please wait...\n")
    vareuFull[,,,,,] <- vareuFull366[,,,1:365,,]

    for(y in year.start:year.end){
        y <- y - year.start + 1
        if(n.days.in.a.year(y) == 366) vareuFull[,,y,60:365,,] <- vareuFull366[,,y,61:366,,] # take the march to december period removing the 29th of February
    }

    rm(vareuFull366)
    gc()

    # convert psl in daily anomalies with the LOESS filter:
    cat("Calculating anomalies. Please wait...\n")
    varPeriodClim <- apply(vareuFull, c(1,2,4,5,6), mean, na.rm=T)

    varPeriodClimLoess <- varPeriodClim
    for(i in n.pos.lat){
        for(j in n.pos.lon){
             my.data <- data.frame(ens.mean=varPeriodClim[1,1,,i,j], day=1:365)
             my.loess <- loess(ens.mean ~ day, my.data, span=0.35)
             varPeriodClimLoess[1,1,,i,j] <- predict(my.loess)
        }
    }

    rm(varPeriodClim)
    gc()

    varPeriodClim2 <- InsertDim(varPeriodClimLoess, 3, n.years)
        
    ## s4.data <- data.frame(s4=pslPeriodClim[1,], day=1:216)
    ## s4.loess <- loess(s4 ~ day, s4.data, span=0.35)
    ## s4.pred <- predict(s4.loess)
        
    vareuFull <- vareuFull - varPeriodClim2    
    #rm(varPeriodClim2)
    gc()

    for(p in WR.period){
       #p=1 # for the debug

       ## load regime data for chosen month:
       varPeriod <- vareuFull[1,1,,pos.period(1,p),,] # select only var data during the chosen period
       varPeriodClim <- varPeriodClim2[1,1,,pos.period(1,p),,]
       varPeriodRel <- varPeriod / varPeriodClim
        
       load(file=paste0(rean.dir,"/",fields.name,"_",my.period[p],"_psl.RData"))
        
       cluster.sequence <- my.cluster$cluster  # old syntax: my.cluster[[p]]$cluster
        
       seasonal.data <- ifelse(length(my.cluster$cluster)/n.years > 33, TRUE, FALSE) # if there are more than 33 days, it means that we are loading sequences of 3 months
       if(seasonal.data == TRUE && p < 13){ # select only the days of the 3-months cluster series inside the target month p
          n.days.period <- length(pos.month.extended(2001,p))  # total number of days in the three months
          
          days.month <- days.month.new <- days.month.full <- c()
          days.month <- which(pos.month.extended(2001,p) == pos.month(2001,p)[1])-1 + 1:length(pos.month(2001,p))
          
          for(y in 1:n.years){
              days.month.new <- days.month + n.days.period*(y-1)
              days.month.full <- c(days.month.full, days.month.new)
              #print(days.month.new)
              #print(days.month)
          }
          
          cluster.sequence <- my.cluster$cluster[days.month.full] # select only the days of the cluster series inside the target month p

       }
        
       wr1 <- which(cluster.sequence == 1)
       wr2 <- which(cluster.sequence == 2)
       wr3 <- which(cluster.sequence == 3)
       wr4 <- which(cluster.sequence == 4)
       
       gc()
        
       cat("Formatting data. Please wait...\n")
       var.melted <- melt(varPeriod[,,,, drop=FALSE], varnames=c("Year","Day","Lat","Lon")) 
       var.meltedRel <- melt(varPeriodRel[,,,, drop=FALSE], varnames=c("Year","Day","Lat","Lon"))
       gc()
        
       cat("Formatting data. Please wait......\n")
       varmat <- unname(acast(var.melted, Year + Day ~ Lat ~ Lon))
       varmatRel <- unname(acast(var.meltedRel, Year + Day ~ Lat ~ Lon))

       varwr1 <- varmat[wr1,,,drop=F]
       varwr2 <- varmat[wr2,,,drop=F]
       varwr3 <- varmat[wr3,,,drop=F]
       varwr4 <- varmat[wr4,,,drop=F]

       varwr1Rel <- varmatRel[wr1,,,drop=F]
       varwr2Rel <- varmatRel[wr2,,,drop=F]
       varwr3Rel <- varmatRel[wr3,,,drop=F]
       varwr4Rel <- varmatRel[wr4,,,drop=F]
        
      
       varwr1mean <- apply(varwr1,c(2,3),mean,na.rm=T)
       varwr2mean <- apply(varwr2,c(2,3),mean,na.rm=T)
       varwr3mean <- apply(varwr3,c(2,3),mean,na.rm=T)
       varwr4mean <- apply(varwr4,c(2,3),mean,na.rm=T)

       varwr1meanRel <- apply(varwr1Rel,c(2,3),mean,na.rm=T)
       varwr2meanRel <- apply(varwr2Rel,c(2,3),mean,na.rm=T)
       varwr3meanRel <- apply(varwr3Rel,c(2,3),mean,na.rm=T)
       varwr4meanRel <- apply(varwr4Rel,c(2,3),mean,na.rm=T)
       
       n.datos <- n.years * n.days.in.a.period(p,2001)
       
       varwrBoth1 <- abind(varmat, varwr1, along = 1)
       pvalue1 <- apply(varwrBoth1, c(2,3), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
       rm(varwrBoth1, varwr1)
       gc()

       varwrBoth2 <- abind(varmat, varwr2, along = 1)
       pvalue2 <- apply(varwrBoth2, c(2,3), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
       rm(varwrBoth2, varwr2)
       gc()
    
       varwrBoth3 <- abind(varmat, varwr3, along = 1)
       pvalue3 <- apply(varwrBoth3, c(2,3), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
       rm(varwrBoth3, varwr3)
       gc()
    
       varwrBoth4 <- abind(varmat, varwr4, along = 1)
       pvalue4 <- apply(varwrBoth4, c(2,3), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
       rm(varwrBoth4, varwr4)
    
       rm(varmat, varmatRel)
       gc()

       #EU <- c(1:which(lon >= lon.max)[1], (which(lon >= 337)[1]:length(lon)))  # restrict area to continental europe only
       #my.brks.var <- seq(-3,3,0.5)
       #my.cols.var <- colorRampPalette(rev(brewer.pal(11,"RdBu")))(length(my.brks.var)-1)    # blue--white--red colors
       #PlotEquiMap2(rescale(varwr3mean[,EU],my.brks.var[1],tail(my.brks.var,1)), lon[EU], lat, filled.continents = FALSE, brks=my.brks.var, cols=my.cols.var, intxlon=10, intylat=10, drawleg=F, cex.lab=1.5, continents.col="black")

       # save all the data necessary to redraw the graphs when we know the right regime:
       save(varwr1mean, varwr2mean, varwr3mean,varwr4mean, pvalue1, pvalue2, pvalue3, pvalue4, varwr1meanRel, varwr2meanRel, varwr3meanRel,varwr4meanRel, file=paste0(rean.dir,"/",fields.name,"_",my.period[p],"_",var.name[var.num],".RData"))
       rm(pvalue1,pvalue2,pvalue3,pvalue4)
       rm(varwr1mean,varwr2mean,varwr3mean,varwr4mean)
       gc()
        
    } # close p on WR.period
    
}



if(fields.name == ECMWF_S4.name){ # for forecasts, we can load only the data for 1 month at time (since each month needs ~20 GB of memory to load all its leadtimes)
   for(p in WR.period){
       #p=1 # for the debug

       start.month <- p
       if(start.month <= 9) {start.month.char <- paste0("0",as.character(start.month))} else {start.month.char <- start.month}

       fields2 <- gsub("\\$STORE_FREQ\\$","daily",fields)
       fields3 <- gsub("\\$VAR_NAME\\$",var.name[var.num],fields2)

       years <- c()

       for(y in year.start:year.end){
            fields4 <- gsub("\\$START_DATE\\$",paste0(y, start.month.char,'01'),fields3)

            if(file.exists(fields4)){
                # beware, in this way it can only take into account leadtimes from 100 to 999, but it is the only way in the SMP machine:
                #char.lead <- nchar(system(paste0("ncks -m ",fields4,"| grep -E -i 'psl size' | cut -d '*' -f1"), intern=T))
                #num.lead <- as.integer(substr(system(paste0("ncks -m ",fields4,"| grep -E -i 'psl size' | cut -d '*' -f1"), intern=T),char.lead-3,char.lead))
                num.lead <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'time dimension' | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T)) #don't work well on the SMP
                #num.memb <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'psl size' | cut -d '=' -f2 | cut -d '*' -f2"), intern=T))

                if(fields == forecast && forecast == ECMWF_S4 && var.name[var.num] == "sfcWind") { # sfcWind daily data of S4 has a different formatting:
                    num.memb <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i '0: '", member.name,"', size' | cut -d '=' -f2 | cut -d ',' -f1"), intern=T))
                } else {
                    num.memb <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i '0: '", member.name,"', size' | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T))                       }
                
                #num.lat  <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'psl size' | cut -d '=' -f2 | cut -d '*' -f3"), intern=T))
                num.lat1 <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'latitude dimension'  | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T))
                num.lat2 <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'lat dimension'  | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T))
                num.lat <- ifelse(length(num.lat2) == 0, num.lat1, num.lat2)

                #num.lon  <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'psl size' | cut -d '=' -f2 | cut -d '*' -f4"), intern=T))
                num.lon1 <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'longitude dimension'  | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T))
                num.lon2 <- as.integer(system(paste0("ncks -m ",fields4,"| grep -E -i 'lon dimension'  | cut -d '=' -f2 | cut -d ' ' -f2"), intern=T))
                num.lon <- ifelse(length(num.lon2) == 0, num.lon1, num.lon2)

                num.var <- system(paste0("ncks -m ",fields4,"| grep -E -i '", var.name[var.num], "'"), intern=T)
                
                if(length(num.lead) != 0 && length(num.memb) != 0 && length(num.lat) != 0 && length(num.lon) != 0 && length(num.var) != 0){
                    if(num.lead == n.leadtimes && num.memb >= n.members) years <- c(years,y)
                }
            }
        }

        n.years.full <- length(years) # years really available for that month
        print(paste0(n.years.full, " available years for ", my.period[p]))
   
        vareuFull <- Load(var = var.name[var.num], exp = list(list(path=fields)), obs = NULL, sdates=paste0(years, start.month.char,'01'), dimnames=list(lon=lon.name, lat=lat.name, member=member.name), nmember=n.members, leadtimemax=n.leadtimes, storefreq='daily', output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max, grid=my.grid, method='bilinear', nprocs=1)$mod

        # convert var in daily anomalies with the LOESS filter:
        cat("Calculating anomalies. Please wait...\n")
        varPeriodClim <- apply(vareuFull, c(1,4,5,6), mean, na.rm=T)
        varPeriodClimLoess <- varPeriodClim

        n.pos.lat <- unname(dim(varPeriodClim)[3])
        n.pos.lon <- unname(dim(varPeriodClim)[4])
    
        for(i in n.pos.lat){
            for(j in n.pos.lon){
                my.data <- data.frame(ens.mean=varPeriodClim[1,,i,j], day=1:n.leadtimes)
                my.loess <- loess(ens.mean ~ day, my.data, span=0.35)
                varPeriodClimLoess[1,,i,j] <- predict(my.loess)
            }
        }

        rm(varPeriodClim)
        gc()

        varPeriodClim2 <- InsertDim(InsertDim(varPeriodClimLoess, 2, n.years.full), 2, n.members)
        
        ## s4.data <- data.frame(s4=pslPeriodClim[1,], day=1:216)
        ## s4.loess <- loess(s4 ~ day, s4.data, span=0.35)
        ## s4.pred <- predict(s4.loess)
        
        vareuFull <- vareuFull - varPeriodClim2    
        rm(varPeriodClim2)
        gc()

        #if(psl=="g500") psleuFull <- psleuFull/9.81   # convert geopotential to geopotential height (in m)
        #if(psl=="psl") psleuFull <- psleuFull/100   # convert MSLP in Pascal to MSLP in hPa
        gc()

        ###### impact maps:
   
        load.month <- p + lead.month
        if(load.month > 12) load.month <- load.month - 12 

        # select only var data during the chosen leadtime, and removing bisestile days!
        varPeriod <- array(NA,c(dim(vareuFull)[1:3],length(pos.period(2001,load.month)),dim(vareuFull)[5:6]))
        
        i <- 1
        for(y in years){
          #y.mod <- y
          #if((p + lead.month) > 12) y.mod <- y + 1
          #pos.per <- pos.period(y.mod, load.month) - ifelse(p == 1, 0, length(c(pos.months.before(y, p-1), pos.period(y, p-1))))

          if(lead.month == 0) pos.per <- 1:ndm(p,y) 
          if(lead.month == 1) pos.per <- ndm(p,y) + 1:ndm(p + 1,y) 
          if(lead.month == 2) pos.per <- ndm(p,y) + ndm(p + 1,y) + 1:ndm(p + 2,y) 
          if(lead.month == 3) pos.per <- ndm(p,y) + ndm(p + 1,y) + ndm(p + 2,y) + 1:ndm(p + 3,y) 
          if(lead.month == 4) pos.per <- ndm(p,y) + ndm(p + 1,y) + ndm(p + 2,y) + ndm(p + 3,y) + 1:ndm(p + 4,y) 
          if(lead.month == 5) pos.per <- ndm(p,y) + ndm(p + 1,y) + ndm(p + 2,y) + ndm(p + 3,y) + ndm(p + 4,y) + 1:ndm(p + 5,y) 
          if(lead.month == 6) pos.per <- ndm(p,y) + ndm(p + 1,y) + ndm(p + 2,y) + ndm(p + 3,y) + ndm(p + 4,y) + ndm(p + 5,y) + 1:ndm(p + 6,y) 

          if(length(pos.per) == 29) {
              #febmod = length(pos.period(y.mod,load.month))
              varPeriod[1,,i,,,] <- vareuFull[1,,i,pos.per[-29],,] 
          } else {
              varPeriod[1,,i,,,] <- vareuFull[1,,i,pos.per,,] 
          }
          
          i <- i+1
        }
        
        gc()

        years.var <- years
        load(file=paste0(forecast.dir,"/",fields.name,"_",my.period[p],"_leadmonth_",lead.month,"_psl.RData")) # load the cluster sequence for a chosen startdate and leadtime        
        #cluster.sequence <- my.cluster[[p]]$cluster #as.vector(my.cluster.array[[p]]) # no bisestile days are left, only the years can be different!
        if(length(years.var) < length(years)) {
            print("Warning: psl years are more than var years. Removing years not in common..")
            ss <- which(is.na(match(years,years.var)))  # remove psl years not present in var data
            my.cluster.array2 <- my.cluster.array[[p]][,-ss,] # (no bisestile days are left in my.cluster.array)
        } else {
            my.cluster.array2 <- my.cluster.array[[p]] # (no bisestile days are left in my.cluster.array)
        }
        
        wr1 <- which(my.cluster.array2 == 1, arr.ind=T)
        wr2 <- which(my.cluster.array2 == 2, arr.ind=T)
        wr3 <- which(my.cluster.array2 == 3, arr.ind=T)
        wr4 <- which(my.cluster.array2 == 4, arr.ind=T)

        var1 <- var2 <- var3 <- var4 <- array(NA,c(dim(varPeriod)))
        
        for (i in 1:dim(wr1)[1]) var1[1, wr1[i,3], wr1[i,2], wr1[i,1],,] <- varPeriod[1, wr1[i,3], wr1[i,2], wr1[i,1],,]
        for (i in 1:dim(wr2)[1]) var2[1, wr2[i,3], wr2[i,2], wr2[i,1],,] <- varPeriod[1, wr2[i,3], wr2[i,2], wr2[i,1],,]
        for (i in 1:dim(wr3)[1]) var3[1, wr3[i,3], wr3[i,2], wr3[i,1],,] <- varPeriod[1, wr3[i,3], wr3[i,2], wr3[i,1],,]
        for (i in 1:dim(wr4)[1]) var4[1, wr4[i,3], wr4[i,2], wr4[i,1],,] <- varPeriod[1, wr4[i,3], wr4[i,2], wr4[i,1],,]

        varwr1mean <- apply(var1, c(5,6), mean, na.rm=T)
        varwr2mean <- apply(var2, c(5,6), mean, na.rm=T)
        varwr3mean <- apply(var3, c(5,6), mean, na.rm=T)
        varwr4mean <- apply(var4, c(5,6), mean, na.rm=T)
        gc()

        n.datos <- n.years.full * n.days.in.a.period(p,2001)
       
        varwrBoth1 <- abind(varPeriod, var1, along = 1)
        pvalue1 <- apply(varwrBoth1, c(5,6), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
        rm(varwrBoth1)
        gc()

        varwrBoth2 <- abind(varPeriod, var2, along = 1)
        pvalue2 <- apply(varwrBoth2, c(5,6), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
        rm(varwrBoth2)
        gc()
    
        varwrBoth3 <- abind(varPeriod, var3, along = 1)
        pvalue3 <- apply(varwrBoth3, c(5,6), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
        rm(varwrBoth3)
        gc()
    
        varwrBoth4 <- abind(varPeriod, var4, along = 1)
        pvalue4 <- apply(varwrBoth4, c(5,6), function(x) t.test(x[1:n.datos],x[(n.datos+1):length(x)])$p.value)
        rm(varwrBoth4)
    
        # save all the data necessary to redraw the graphs when we know the right regime:
        save(varwr1mean, varwr2mean, varwr3mean, varwr4mean, pvalue1, pvalue2, pvalue3, pvalue4, file=paste0(forecast.dir,"/",fields.name,"_",my.period[p],"_leadmonth_",lead.month,"_",var.name[var.num],".RData"))
       
        rm(varwr1mean,varwr2mean,varwr3mean,varwr4mean)
        gc()
            
    } # close p on period
} # close if on data type




if(fields.name == ECMWF_monthly.name){
        # Load 6-hourly psl data in the forecast case:    
        psleuFull <-array(NA, c(n.sdates*n.years, n.leadtimes, n.pos.lat, n.pos.lon))  # daily order: 19940102 19950102 ... 20130102 19940109 19950109 ... 20130109 ...
        n.leadtimes <- length(leadtime)
        sdates <- weekly.seq(forecast.year,mes,day)
        n.sdates <- length(sdates)
        
        for (startdate in 1:n.sdates){
            for(lday in leadtime){
                var <- Load(var = psl, exp = NULL, obs = list(list(path=fields)), paste0(year.start:year.end, substr(sdates[startdate],5,6), substr(sdates[startdate],7,8) ), storefreq = 'daily', leadtimemin = lday*4-3, leadtimemax = lday*4, output = 'lonlat', latmin = lat.min, latmax = lat.max, lonmin = lon.min, lonmax = lon.max, nprocs=1)
                
                mean.lday <- apply(var$obs, c(3,5,6), mean, na.rm=T)
                rm(var)
                gc()
                
                psleuFull[(1+(startdate-1)*n.years):(startdate*n.years), lday-leadtime[1]+1,,] <- mean.lday
                rm(mean.lday)
                gc()
            }
        }
    }    



cat("Finished!\n")   

